#!/usr/bin/python
# -*- coding: utf-8 -*-

#       Config.py
#       
#       Copyright 2011 TuxGasy <tuxgasy@tuxfamily.org>
#       
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#       
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#       
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

import os
import io
import ConfigParser

DIR_USER_CONFIG = os.path.expanduser('~/.glda')
FILE_USER_CONFIG = DIR_USER_CONFIG+'/glda.conf'

class Config(ConfigParser.RawConfigParser) :
	
	def createFileConfig(self) :
		""" Create configuration file ~/.glda/glda.conf """
		
		default_config = """
[general]
devise = €

[database]
db_file = %s/glda.db

[view]
window_width = 950
window_height = 600
sort_category = 
sort_product_added = 
sort_product = 
show_column_category_in_cart = true
show_column_subcategory_in_cart = true
show_column_price = true
show_column_quantity = true
show_total_quantity = true
show_total_price = true

[print]
font_name = Monospace 10.5
color = #000000
show_title = true
show_page_number = true
table_border = 1
show_column_category = true
show_column_subcategory = true
show_column_price = true
show_column_quantity = true
show_line_total = true
		"""%DIR_USER_CONFIG
		
		self.readfp(io.BytesIO(default_config))
		
		# Writing our configuration file to ~/.glda/glda.conf
		self.write(open(FILE_USER_CONFIG, 'wb'))
	
	def set(self, section, option, value) :
		ConfigParser.RawConfigParser.set(self, section, option, value)
		self.save()
	
	def setint(self, section, option, value) :
		self.set(section, option, str(value))
	
	def setfloat(self, section,  option, value) :
		self.set(section, option, str(value))
	
	def setboolean(self, section, option, value) :
		if value :
			self.set(section, option, "true")
		else :
			self.set(section, option, "false")
	
	def save(self) :
		# Writing our configuration file to ~/.glda/glda.conf
		self.write(open(FILE_USER_CONFIG, 'wb'))
	
	def __init__(self) :
		ConfigParser.RawConfigParser.__init__(self)
		
		# create ~/.glda if not exist
		if not os.access(DIR_USER_CONFIG, os.F_OK | os.W_OK) :
			os.mkdir(DIR_USER_CONFIG)
		
		# create ~/.glda/glda.conf if not exist
		if not os.access(FILE_USER_CONFIG, os.F_OK | os.W_OK) :
			self.createFileConfig()
		else :
			self.read(FILE_USER_CONFIG)
