#!/usr/bin/python
# -*- coding: utf-8 -*-

#       Db.py
#       
#       Copyright 2011 TuxGasy <tuxgasy@tuxfamily.org>
#       
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#       
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#       
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

import sqlite3

class Db :
	""" Database for products """
	
	conn = None
	curs = None
	db_file = None
	
	def connect(self) :
		self.conn = sqlite3.connect(self.db_file)
		self.conn.row_factory = sqlite3.Row
		self.curs = self.conn.cursor()
	
	def close(self) :
		# save database
		self.conn.commit()
		# close the cursor
		self.curs.close()
		# close the connection
		self.conn.close()
	
	def set_category(self, name) :
		"""set category """
		self.connect()
		self.curs.execute("INSERT INTO category (name) VALUES (?)", (name,))
		id = self.curs.lastrowid
		self.close()
		return id
	
	def update_category(self, id, name) :
		""" update category """
		self.connect()
		self.curs.execute("UPDATE category SET name=? WHERE id=?", (name, id))
		self.close()
	
	def set_subcategory(self, id_category, name) :
		"""set subcategory """
		self.connect()
		self.curs.execute("INSERT INTO subcategory (id_category, name) VALUES (?, ?)", (id_category, name))
		id = self.curs.lastrowid
		self.close()
		return id
	
	def update_subcategory(self, id, name) :
		"""update subcategory """
		self.connect()
		self.curs.execute("UPDATE subcategory SET name=? WHERE id=?", (name, id))
		self.close()
	
	def set_article(self, id_category, id_subcategory, name, price) :
		""" set article """
		self.connect()
		self.curs.execute("INSERT INTO articles (id_subcategory, id_category, name, price) VALUES (?, ?, ?, ?)",
												(id_subcategory, id_category, name, price))
		id = self.curs.lastrowid
		self.close()
		return id
	
	def update_article(self, id, name, price) :
		""" update article """
		self.connect()
		self.curs.execute("UPDATE articles SET name=? , price=? WHERE id=?", (name, price, id))
		self.close()
	
	def get_all_category(self) :
		""" get all category """
		self.connect()
		self.curs.execute("SELECT * FROM category")
		category = self.curs.fetchall()
		self.close()
		return category
	
	def get_category(self, id) :
		""" get category """
		self.connect()
		self.curs.execute("SELECT * FROM category WHERE id=?", (id,))
		category = self.curs.fetchone()
		self.close()
		return category
	
	def remove_category(self, id) :
		""" remove category """
		self.connect()
		self.curs.execute("DELETE FROM category WHERE id=?", (id,))
		self.close()
	
	def get_all_subcategory_by_category(self, id_category) :
		""" get all subcategory by category """
		self.connect()
		self.curs.execute("SELECT * FROM subcategory WHERE id_category=?", (id_category,))
		subcategory = self.curs.fetchall()
		self.close()
		return subcategory
	
	def get_subcategory(self, id) :
		""" get subcategory """
		self.connect()
		self.curs.execute("SELECT * FROM subcategory WHERE id=?", (id,))
		subcategory = self.curs.fetchone()
		self.close()
		return subcategory
	
	def remove_subcategory(self, id) :
		""" remove subcategory """
		self.connect()
		self.curs.execute("DELETE FROM subcategory WHERE id=?", (id,))
		self.close()
	
	def get_all_articles_by_subcategory(self, id_subcategory) :
		""" get all articles by subcategory """
		self.connect()
		self.curs.execute("SELECT * FROM articles WHERE id_subcategory=?", (id_subcategory,))
		articles = self.curs.fetchall()
		self.close()
		return articles
	
	def get_article(self, id) :
		""" get article """
		self.connect()
		self.curs.execute("SELECT * FROM articles WHERE id=?", (id,))
		article = self.curs.fetchone()
		self.close()
		return article
	
	def remove_article(self, id) :
		""" remove article """
		self.connect()
		self.curs.execute("DELETE FROM articles WHERE id=?", (id,))
		self.close()
	
	def new_cart(self, name) :
		""" set cart """
		self.connect()
		self.curs.execute("INSERT INTO cart (name) VALUES (?)", (name,))
		id = self.curs.lastrowid
		self.close()
		return id
	
	def get_all_cart(self) :
		""" get all cart """
		self.connect()
		self.curs.execute("SELECT * FROM cart")
		cart = self.curs.fetchall()
		self.close()
		return cart
	
	def get_cart(self, id) :
		""" get cart """
		self.connect()
		self.curs.execute("SELECT * FROM cart WHERE id=?", (id,))
		cart = self.curs.fetchone()
		self.close()
		return cart
	
	def get_purchase(self, id) :
		""" get purchase """
		self.connect()
		self.curs.execute("SELECT * FROM purchase WHERE id=?", (id,))
		purchase = self.curs.fetchone()
		self.close()
		return purchase
	
	def get_purchases_by_article(self, id_article) :
		""" get purchase by article id """
		self.connect()
		self.curs.execute("SELECT * FROM purchase WHERE id_article=?", (id_article,))
		purchases = self.curs.fetchall()
		self.close()
		return purchases
	
	def get_purchase_by_article_and_cart(self, id_cart, id_article) :
		""" get purchase by article id and cart id """
		self.connect()
		self.curs.execute("SELECT * FROM purchase WHERE id_cart=? AND id_article=?", (id_cart, id_article,))
		purchase = self.curs.fetchone()
		self.close()
		return purchase
	
	def set_purchase(self, id_cart, id_article, quantity) :
		""" set purchase """
		self.connect()
		self.curs.execute("INSERT INTO purchase (id_cart, id_article, quantity) VALUES (?, ?, ?)", (id_cart, id_article, quantity))
		id = self.curs.lastrowid
		self.close()
		return id
	
	def update_quantity_purchase(self, id, quantity) :
		""" update quantity """
		self.connect()
		self.curs.execute("UPDATE purchase SET quantity=? WHERE id=?", (quantity, id))
		self.close()
	
	def get_all_purchase(self, id_cart) :
		""" get all purchase for a cart """
		self.connect()
		self.curs.execute("SELECT * FROM purchase WHERE id_cart=?", (id_cart,))
		purchases = self.curs.fetchall()
		self.close()
		return purchases
	
	def get_all_purchase_with_limit(self, id_cart, limit, offset) :
		""" get all purchase for a cart """
		self.connect()
		self.curs.execute("SELECT * FROM purchase WHERE id_cart=? LIMIT ? OFFSET ?", (id_cart, limit, offset))
		purchases = self.curs.fetchall()
		self.close()
		return purchases
	
	def remove_purchase(self, id) :
		""" remove purchase """
		self.connect()
		self.curs.execute("DELETE FROM purchase WHERE id=?", (id,))
		self.close()
	
	def __init__(self, db_file) :
		""" init database """
		self.db_file = db_file
		self.connect()
		
		# check if articles'table exist
		try:
			self.curs.execute("SELECT * FROM articles")
		except sqlite3.OperationalError :
			""" create all tables """
			self.curs.executescript("""
				CREATE TABLE articles(
					id INTEGER PRIMARY KEY AUTOINCREMENT,
					id_category INTEGER NOT NULL,
					id_subcategory INTEGER NOT NULL,
					name VARCHAR NOT NULL,
					price FLOAT NOT NULL,
					position INTEGER DEFAULT 0
				);
				
				CREATE TABLE subcategory(
					id INTEGER PRIMARY KEY AUTOINCREMENT,
					id_category INTEGER NOT NULL,
					name VARCHAR NOT NULL,
					position INTEGER DEFAULT 0
				);
				
				CREATE TABLE category(
					id INTEGER PRIMARY KEY AUTOINCREMENT,
					name VARCHAR NOT NULL,
					position INTEGER DEFAULT 0
				);
				
				CREATE TABLE cart(
					id INTEGER PRIMARY KEY AUTOINCREMENT,
					name VARCHAR NOT NULL,
					position INTEGER DEFAULT 0
				);
				
				INSERT INTO cart (name) VALUES ("new_cart");
				
				CREATE TABLE purchase(
					id INTEGER PRIMARY KEY AUTOINCREMENT,
					id_cart INTEGER NOT NULL,
					id_article INTEGER NOT NULL,
					quantity INTEGER NOT NULL,
					position INTEGER DEFAULT 0
				);
			""")
		
		self.close()
