#!/usr/bin/python
# -*- coding: utf-8 -*-

#       Produts.py
#       
#       Copyright 2008 TuxGasy <tuxgasy@tuxfamily.org>
#       
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#       
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#       
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

import os
import xml.dom.minidom

class Products :
	
	dom = None
	
	def saveToFile(self, fileXml) :
		" Save to file "
		f = open(fileXml, 'w+')
		f.write(self.dom.toprettyxml(encoding='UTF-8'))
		f.close()
	
	def getTextNode(self, nodelist) :
		" Return the data's node "
		rc = ""
		for node in nodelist :
			if node.nodeType == node.TEXT_NODE :
				rc = rc+node.data
		
		return rc
	
	def getCategory(self, id=0) :
		" Return category's informations as a list [Name, Reference, Dom category] "
		try :
			categoryDom = self.dom.getElementsByTagName('category')[id]
			categoryInfos = [categoryDom.getAttributeNS(None, 'name'), categoryDom.getAttributeNS(None, 'ref'), categoryDom]
		except IndexError :
			categoryInfos = None
		
		return categoryInfos
	
	def getSubCategory(self, categoryDom, id=0) :
		" Return subcategory's informations as a list [Name, Reference, Dom subcategory] "
		try :
			subCategoryDom = categoryDom.getElementsByTagName('subcategory')[id]
			subCategoryInfos = [subCategoryDom.getAttributeNS(None, 'name'), subCategoryDom.getAttributeNS(None, 'ref'), subCategoryDom]
		except IndexError :
			subCategoryInfos = None
		
		return subCategoryInfos
	
	def getProducts(self, subCategoryDom, id=0) :
		" Return product's informations as a list [Name, Price, Reference] "
		try :
			productDom = subCategoryDom.getElementsByTagName('product')[id]
			productNameDom = productDom.getElementsByTagName('name')[0]
			productPriceDom = productDom.getElementsByTagName('price')[0]
			productInfos = [self.getTextNode(productNameDom.childNodes), self.getTextNode(productPriceDom.childNodes), productDom.getAttributeNS(None, 'ref')]
		except IndexError :
			productInfos = None
		
		return productInfos
	
	def getProductWithReferences(self, categoryReference, subCategoryReference, productReference) :
		" Return product's informations as a list [Category's name, Subcategory's name, Product's name, Product's price] "
		# search category
		i = 0
		category = self.getCategory(i)
		while category != None and category[1] != categoryReference :
			i = i+1
			category = self.getCategory(i)
		
		# No subcategory
		if category == None :
			return None
		
		i = 0
		subcategory = self.getSubCategory(category[2], i)
		while subcategory != None and subcategory[1] != subCategoryReference :
			i = i+1
			subcategory = self.getSubCategory(category[2], i)
		
		# No product
		if subcategory == None :
			return None
		
		i = 0
		products = self.getProducts(subcategory[2], i)
		while products != None and products[2] != productReference :
			i = i+1
			products = self.getProducts(subcategory[2], i)
		
		if products != None :
			productInfos = [category[0], subcategory[0], products[0], products[1]]
			return productInfos
		else :
			return None
	
	def __init__(self, fileXml) :
		if not os.access(fileXml, os.F_OK | os.W_OK) :
			self.dom = xml.dom.minidom.parseString('<?xml version="1.0" encoding="UTF-8"?><lda />')
			self.saveToFile(fileXml)
		else :
			self.dom = xml.dom.minidom.parse(fileXml)
