#!/usr/bin/python
# -*- coding: utf-8 -*-

#       glda.py
#       
#       Copyright 2011 TuxGasy <tuxgasy@tuxfamily.org>
#       
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#       
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#       
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

import pygtk
pygtk.require("2.0")
import gtk
import gtk.glade

import os
import pango
import math

from Db import *
from Config import *

from xml2sqlite import *

appInfos = {}
appInfos['name'] = "Glda"
appInfos['version'] = "0.7"
appInfos['website'] = "http://glda.tuxfamily.org"
appInfos['authors'] = ['TuxGasy <tuxgasy@tuxfamily.org>']
appInfos['documenters'] = ['fanfan la tulipe <olivier.fandard1@tele2.fr>']
appInfos['artists'] = ['Isabelle Stévant <isabelle.stevant@gmail.com>']
appInfos['icon'] = "/usr/share/pixmaps/glda.png"
if not os.access(appInfos['icon'], os.F_OK) :
	appInfos['icon'] = "pixmaps/glda.png"
appInfos['license'] = "/usr/share/doc/glda/copyright"
if not os.access(appInfos['license'], os.F_OK) :
	appInfos['license'] = "doc/copyright"

GLDA_GLADE_FILE = "/usr/share/glda/glda.glade"
if not os.access(GLDA_GLADE_FILE, os.F_OK) :
	GLDA_GLADE_FILE = "src/glda.glade"

##################### GOBAL VARIABLE FOR COLUMN NUMBER #################
#
COLUMN_CATEGORY = 0
COLUMN_CATEGORY_ID = 1
COLUMN_SUBCATEGORY_ID = 2

COLUMN_ARTICLE = 0
COLUMN_ARTICLE_PRICE = 1
COLUMN_ARTICLE_ID = 2

COLUMN_PURCHASE_CATEGORY = 0
COLUMN_PURCHASE_SUBCATEGORY = 1
COLUMN_PURCHASE_ARTICLE = 2
COLUMN_PURCHASE_QUANTITY = 3
COLUMN_PURCHASE_PRICE = 4
COLUMN_PURCHASE_ID = 5

COLUMN_STORE_CATEGORY = 0
COLUMN_STORE_PRICE = 1
COLUMN_STORE_CATEGORY_ID = 2
COLUMN_STORE_SUBCATEGORY_ID = 3
COLUMN_STORE_ARTICLE_ID = 4
#
########################################################################

##################### GOBAL VARIABLE FOR PRINT #########################
#
COLUMN_ARTICLE_SIZE = 40
COLUMN_QUANTITY_SIZE = 10
COLUMN_PRICE_SIZE = 10
#
########################################################################

class mainWindow() :
	
	def __init__(self, config, db, appInfos) :
		self.config = config
		self.db = db
		self.appInfos = appInfos
		self.current_cart_id = 1
		
		self.widgets = gtk.glade.XML(GLDA_GLADE_FILE, "mainWindow")
		events = {
			"gtk_main_quit" : self.on_quit,
			"on_menuitem_quit_activate" : self.on_quit,
			"on_toolbutton_quit_clicked" : self.on_quit,
			"on_menuitem_store_manage_activate" : self.on_manage_store,
			"on_menuitem_preferences_activate" : self.on_preferences,
			"on_toolbutton_preferences_clicked" : self.on_preferences,
			"on_menuitem_about_activate" : self.on_about,
			"on_menuitem_add_activate" : self.on_add_articles_on_cart_activate,
			"on_treeview_articles_row_activated" : self.on_add_articles_on_cart_row_activated,
			"on_button_add_articles_on_cart_clicked" : self.on_add_articles_on_cart_clicked,
			"on_menuitem_remove_activate" : self.on_remove_articles_on_cart_activate,
			"on_button_remove_on_cart_clicked_clicked" : self.on_remove_articles_on_cart,
			"on_button_print_clicked" : self.on_print
		}
		self.widgets.signal_autoconnect(events)
		
		mainWindow = self.widgets.get_widget('mainWindow')
		mainWindow.set_icon_from_file(appInfos['icon'])
		mainWindow.set_title("new_cart : %s"%self.appInfos['name'])
		width = config.getint("view", "window_width")
		height = config.getint("view", "window_height")
		mainWindow.resize(width, height)
		
		self.treeview_category()
		self.treeview_articles()
		self.treeview_cart()
		
		self.refresh_category()
		self.refresh_cart()
		
		self.widgets.get_widget('statusbar').push(0, "Glda (Gtk Liste D'Achat) version "+self.appInfos['version'])
		
		self.refresh_display()
	
	#
	#	Create treeview for category
	def treeview_category(self) :
		tree_store_category = gtk.TreeStore(str, int, int)
		treeview_category = self.widgets.get_widget('treeview_category')
		treeview_category.set_model(tree_store_category)
		treeview_category.set_search_column(COLUMN_CATEGORY)
		
		tree_selection = treeview_category.get_selection()
		tree_selection.set_mode(gtk.SELECTION_SINGLE)
		tree_selection.connect("changed", self.on_select_category, None)
		
		cellRenderer = gtk.CellRendererText()
		column = gtk.TreeViewColumn("Categories", cellRenderer)
		column.add_attribute(cellRenderer, "text", COLUMN_CATEGORY)
		column.set_sort_column_id(COLUMN_CATEGORY)
		treeview_category.append_column(column)
		
		cellRenderer = gtk.CellRendererText()
		column = gtk.TreeViewColumn("id-category", cellRenderer)
		column.add_attribute(cellRenderer, "text", COLUMN_CATEGORY_ID)
		column.set_visible(False)
		treeview_category.append_column(column)
		
		cellRenderer = gtk.CellRendererText()
		column = gtk.TreeViewColumn("id-subcategory", cellRenderer)
		column.add_attribute(cellRenderer, "text", COLUMN_SUBCATEGORY_ID)
		column.set_visible(False)
		treeview_category.append_column(column)
	
	#
	#	Create treeview for articles
	def treeview_articles(self) :
		list_store_articles = gtk.ListStore(str, str, int)
		treeview_articles = self.widgets.get_widget('treeview_articles')
		treeview_articles.set_model(list_store_articles)
		treeview_articles.set_search_column(COLUMN_ARTICLE)
		#treeview_articles.connect("row-activated", self.on_double_click_article, None)
		
		tree_selection = treeview_articles.get_selection()
		tree_selection.set_mode(gtk.SELECTION_MULTIPLE)
		
		cellRenderer = gtk.CellRendererText()
		column = gtk.TreeViewColumn("Articles", cellRenderer)
		column.add_attribute(cellRenderer, "text", COLUMN_ARTICLE)
		column.set_sort_column_id(COLUMN_ARTICLE)
		treeview_articles.append_column(column)
		
		cellRenderer = gtk.CellRendererText()
		column = gtk.TreeViewColumn("Prix", cellRenderer)
		column.add_attribute(cellRenderer, "text", COLUMN_ARTICLE_PRICE)
		column.set_sort_column_id(COLUMN_ARTICLE_PRICE)
		treeview_articles.append_column(column)
		
		cellRenderer = gtk.CellRendererText()
		column = gtk.TreeViewColumn("id-article", cellRenderer)
		column.add_attribute(cellRenderer, "text", COLUMN_ARTICLE_ID)
		column.set_visible(False)
		treeview_articles.append_column(column)
	
	#
	#	Create treeview for cart
	def treeview_cart(self) :
		list_store_cart = gtk.ListStore(str, str, str, int, str, float, int)
		treeview_cart = self.widgets.get_widget('treeview_cart')
		treeview_cart.set_model(list_store_cart)
		treeview_cart.set_search_column(COLUMN_PURCHASE_ARTICLE)
		
		tree_selection = treeview_cart.get_selection()
		tree_selection.set_mode(gtk.SELECTION_MULTIPLE)
		
		cellRenderer = gtk.CellRendererText()
		column = gtk.TreeViewColumn("Categories", cellRenderer)
		column.add_attribute(cellRenderer, "text", COLUMN_PURCHASE_CATEGORY)
		column.set_sort_column_id(COLUMN_PURCHASE_CATEGORY)
		treeview_cart.append_column(column)
		
		cellRenderer = gtk.CellRendererText()
		column = gtk.TreeViewColumn("Sous-categories", cellRenderer)
		column.add_attribute(cellRenderer, "text", COLUMN_PURCHASE_SUBCATEGORY)
		column.set_sort_column_id(COLUMN_PURCHASE_SUBCATEGORY)
		treeview_cart.append_column(column)
		
		cellRenderer = gtk.CellRendererText()
		column = gtk.TreeViewColumn("Articles", cellRenderer)
		column.add_attribute(cellRenderer, "text", COLUMN_PURCHASE_ARTICLE)
		column.set_sort_column_id(COLUMN_PURCHASE_ARTICLE)
		treeview_cart.append_column(column)
		
		adjustment = gtk.Adjustment(0, 1, 100, 1, 10, 0)        
		cellRenderer = gtk.CellRendererSpin()
		cellRenderer.set_property("editable", True)
		cellRenderer.set_property("adjustment", adjustment)
		cellRenderer.connect("edited", self.on_change_quantity_purchase)
		column = gtk.TreeViewColumn("Quantité", cellRenderer)
		column.add_attribute(cellRenderer, "text", COLUMN_PURCHASE_QUANTITY)
		column.set_sort_column_id(COLUMN_PURCHASE_QUANTITY)
		treeview_cart.append_column(column)
		
		cellRenderer = gtk.CellRendererText()
		column = gtk.TreeViewColumn("Prix", cellRenderer)
		column.add_attribute(cellRenderer, "text", COLUMN_PURCHASE_PRICE)
		column.set_sort_column_id(COLUMN_PURCHASE_PRICE)
		treeview_cart.append_column(column)
		
		cellRenderer = gtk.CellRendererText()
		column = gtk.TreeViewColumn("id-purchase", cellRenderer)
		column.add_attribute(cellRenderer, "text", COLUMN_PURCHASE_ID)
		column.set_visible(False)
		treeview_cart.append_column(column)
	
	#
	#	Refresh category list
	def refresh_category(self) :
		treeview_category = self.widgets.get_widget('treeview_category')
		tree_store_category = treeview_category.get_model()
		tree_store_category.clear()
		
		all_category = self.db.get_all_category()
		for category in all_category :
			iter = tree_store_category.append(None)
			tree_store_category.set(iter,
						COLUMN_CATEGORY, category["name"],
						COLUMN_CATEGORY_ID, category["id"],
						COLUMN_SUBCATEGORY_ID, 0
			)
			
			all_subcategory = self.db.get_all_subcategory_by_category(category["id"])
			for subcategory in all_subcategory :
				iter_child = tree_store_category.append(iter)
				tree_store_category.set(iter_child,
							COLUMN_CATEGORY, subcategory["name"],
							COLUMN_CATEGORY_ID, subcategory["id_category"],
							COLUMN_SUBCATEGORY_ID, subcategory["id"]
				)
	
	#
	#	Refresh cart list
	def refresh_cart(self) :
		treeview_cart = self.widgets.get_widget('treeview_cart')
		tree_store_cart = treeview_cart.get_model()
		tree_store_cart.clear()
		
		total_price = 0.00
		total_quantity = 0
		all_purchase = self.db.get_all_purchase(self.current_cart_id)
		for purchase in all_purchase :
			article = self.db.get_article(purchase["id_article"])
			subcategory = self.db.get_subcategory(article["id_subcategory"])
			category = self.db.get_category(article["id_category"])
			
			price = float(article["price"])*float(purchase["quantity"])
			iter = tree_store_cart.append(None)
			tree_store_cart.set(iter,
						COLUMN_PURCHASE_CATEGORY, category["name"],
						COLUMN_PURCHASE_SUBCATEGORY, subcategory["name"],
						COLUMN_PURCHASE_ARTICLE, article["name"],
						COLUMN_PURCHASE_QUANTITY, purchase["quantity"],
						COLUMN_PURCHASE_PRICE, "%.2f"%price,
						COLUMN_PURCHASE_ID, purchase["id"]
			)
			
			total_price = total_price + article["price"] * purchase["quantity"]
			total_quantity = total_quantity + purchase["quantity"]
		
		devise = self.config.get('general', 'devise')
		self.widgets.get_widget('label_total_articles').set_text("%d"%total_quantity)
		self.widgets.get_widget('label_total_price').set_text("%.2f %s"%(total_price, devise))
	
	#
	#	Refresh display
	def refresh_display(self) :
		treeview_cart = self.widgets.get_widget("treeview_cart")
		treeview_articles = self.widgets.get_widget("treeview_articles")
		
		show_column_category_in_cart = self.config.getboolean('view', 'show_column_category_in_cart')
		column = treeview_cart.get_column(COLUMN_PURCHASE_CATEGORY)
		column.set_visible(show_column_category_in_cart)
		
		show_column_subcategory_in_cart = self.config.getboolean('view', 'show_column_subcategory_in_cart')
		column = treeview_cart.get_column(COLUMN_PURCHASE_SUBCATEGORY)
		column.set_visible(show_column_subcategory_in_cart)
		
		show_column_price = self.config.getboolean('view', 'show_column_price')
		column = treeview_cart.get_column(COLUMN_PURCHASE_PRICE)
		column.set_visible(show_column_price)
		column = treeview_articles.get_column(COLUMN_ARTICLE_PRICE)
		column.set_visible(show_column_price)
		
		show_column_quantity = self.config.getboolean('view', 'show_column_quantity')
		column = treeview_cart.get_column(COLUMN_PURCHASE_QUANTITY)
		column.set_visible(show_column_quantity)
		
		if self.config.getboolean('view', 'show_total_quantity') :
			self.widgets.get_widget('label_total_articles0').show()
			self.widgets.get_widget('label_total_articles').show()
		else :
			self.widgets.get_widget('label_total_articles0').hide()
			self.widgets.get_widget('label_total_articles').hide()
		
		if self.config.getboolean('view', 'show_total_price') :
			self.widgets.get_widget('label_total_price0').show()
			self.widgets.get_widget('label_total_price').show()
		else :
			self.widgets.get_widget('label_total_price0').hide()
			self.widgets.get_widget('label_total_price').hide()
	
	#
	#	Refresh articles list on select subcategory
	def on_select_category(self, widget, data) :
		treeview_articles = self.widgets.get_widget('treeview_articles')
		tree_store_articles = treeview_articles.get_model()
		tree_store_articles.clear()
		
		treeview_category = self.widgets.get_widget('treeview_category')
		tree_selection = treeview_category.get_selection()
		tree_store_category, iter_selected = tree_selection.get_selected()
		if iter_selected != None :
			subcategory_id = tree_store_category.get(iter_selected, COLUMN_SUBCATEGORY_ID)[0]
			if subcategory_id != 0 :
				all_articles = self.db.get_all_articles_by_subcategory(subcategory_id)
				for article in all_articles :
					iter = tree_store_articles.append(None)
					tree_store_articles.set(iter,
								COLUMN_ARTICLE, article["name"],
								COLUMN_ARTICLE_PRICE, "%.2f"%article["price"],
								COLUMN_ARTICLE_ID, article["id"]
					)
	
	#
	#	Add articles on cart
	def on_add_articles_on_cart_activate(self, widget) :
		self.add_articles_on_cart()
	
	def on_add_articles_on_cart_row_activated(self, treeview, path, column) :
		self.add_articles_on_cart()
	
	def on_add_articles_on_cart_clicked(self, widget, data=None) :
		self.add_articles_on_cart()
	
	def add_articles_on_cart(self) :
		treeview_articles = self.widgets.get_widget('treeview_articles')
		tree_selection = treeview_articles.get_selection()
		
		tree_store_articles, rows_selected = tree_selection.get_selected_rows()
		nb_rows_selected = tree_selection.count_selected_rows()
		if nb_rows_selected > 0 :
			for row in rows_selected :
				iter = tree_store_articles.get_iter(row)
				id_article = tree_store_articles.get(iter, COLUMN_ARTICLE_ID)[0]
				purchase = self.db.get_purchase_by_article_and_cart(self.current_cart_id, id_article)
				if purchase == None :
					self.db.set_purchase(self.current_cart_id, id_article, 1)
				else :
					self.db.update_quantity_purchase(purchase["id"], purchase["quantity"]+1)
		
			self.refresh_cart()
	
	#
	#	Remove articles on cart
	def on_remove_articles_on_cart_activate(self, widget) :
		self.remove_articles_on_cart()
	
	def on_remove_articles_on_cart(self, widget, data=None) :
		self.remove_articles_on_cart()
	
	def remove_articles_on_cart(self) :
		treeview_cart = self.widgets.get_widget('treeview_cart')
		tree_selection = treeview_cart.get_selection()
		
		tree_store_cart, rows_selected = tree_selection.get_selected_rows()
		nb_rows_selected = tree_selection.count_selected_rows()
		if nb_rows_selected > 0 :
			for row in rows_selected :
				iter = tree_store_cart.get_iter(row)
				id_purchase = tree_store_cart.get(iter, COLUMN_PURCHASE_ID)[0]
				self.db.remove_purchase(id_purchase)
			
			self.refresh_cart()
	
	def on_change_quantity_purchase(self, widget, path, value) :
		try :
			# except ValueError if value is not a int or float
			new_value = int(value)
			treeview_cart = self.widgets.get_widget('treeview_cart')
			tree_store_cart = treeview_cart.get_model()
			tree_selection = treeview_cart.get_selection()
			
			tree_store_cart[path][COLUMN_PURCHASE_QUANTITY] = new_value
			id_purchase = tree_store_cart[path][COLUMN_PURCHASE_ID]
			
			self.db.update_quantity_purchase(id_purchase, new_value)
			
			self.refresh_cart()
			tree_selection.select_path(path)
		except ValueError :
			pass
		
	
	#	On quit app
	def on_quit(self, widget, data=None) :
		gtk.main_quit()
	
	#
	####################################################################
	################## Start Preferences Dialog ########################
	####################################################################
	#	Callbacks
	def on_change_devise(self, entry, data=None) :
		self.config.set("general", "devise", entry.get_text())
	
	def on_change_check_button_show_column_category_in_cart(self, check_button) :
		self.config.setboolean('view', 'show_column_category_in_cart', check_button.get_active())
	
	def on_change_check_button_show_column_subcategory_in_cart(self, check_button) :
		self.config.setboolean('view', 'show_column_subcategory_in_cart', check_button.get_active())
	
	def on_change_check_button_show_column_price(self, check_button) :
		self.config.setboolean('view', 'show_column_price', check_button.get_active())
	
	def on_change_check_button_show_column_quantity(self, check_button) :
		self.config.setboolean('view', 'show_column_quantity', check_button.get_active())
	
	def on_change_check_button_show_total_quantity(self, check_button) :
		self.config.setboolean('view', 'show_total_quantity', check_button.get_active())
	
	def on_change_check_button_total_price(self, check_button) :
		self.config.setboolean('view', 'show_total_price', check_button.get_active())
	
	def on_change_print_police(self, font_button) :
		self.config.set('print', 'font_name', font_button.get_font_name())
	
	def on_change_print_color(self, color_button) :
		#print color_button.get_color()
		#print color_button.get_color().to_string()
		self.config.set('print', 'color', str(color_button.get_color()))
	
	def on_change_checkbutton_show_title(self, check_button) :
		self.config.setboolean('print', 'show_title', check_button.get_active())
	
	def on_change_show_page_number(self, check_button) :
		self.config.setboolean('print', 'show_page_number', check_button.get_active())
	
	def on_change_table_border(self, spin_button) :
		self.config.setint('print', 'table_border', spin_button.get_value_as_int())
	
	def on_change_show_print_column_price(self, check_button) :
		self.config.setboolean('print', 'show_column_price', check_button.get_active())
	
	def on_change_show_print_column_quantity(self, check_button) :
		self.config.setboolean('print', 'show_column_quantity', check_button.get_active())
	
	def on_change_show_line_total(self, check_button) :
		self.config.setboolean('print', 'show_line_total', check_button.get_active())
	
	#
	#	Preferences Dialog
	def on_preferences(self, widget, data=None) :
		widgets = gtk.glade.XML(GLDA_GLADE_FILE, "preferencesDialog")
		dialog = widgets.get_widget('preferencesDialog')
		dialog.set_transient_for(self.widgets.get_widget('mainWindow'))
		
		## Set default option for General Tab
		entry_devise = widgets.get_widget('entry_devise')
		entry_devise.set_text(self.config.get('general', 'devise'))
		
		checkbutton_show_column_category_in_cart = widgets.get_widget('checkbutton_show_column_category_in_cart')
		checkbutton_show_column_category_in_cart.set_active(self.config.getboolean('view', 'show_column_category_in_cart'))
		
		checkbutton_show_column_subcategory_in_cart = widgets.get_widget('checkbutton_show_column_subcategory_in_cart')
		checkbutton_show_column_subcategory_in_cart.set_active(self.config.getboolean('view', 'show_column_subcategory_in_cart'))
		
		checkbutton_show_column_price = widgets.get_widget('checkbutton_show_column_price')
		checkbutton_show_column_price.set_active(self.config.getboolean('view', 'show_column_price'))
		
		checkbutton_show_column_quantity = widgets.get_widget('checkbutton_show_column_quantity')
		checkbutton_show_column_quantity.set_active(self.config.getboolean('view', 'show_column_quantity'))
		
		checkbutton_show_total_quantity = widgets.get_widget('checkbutton_show_total_quantity')
		checkbutton_show_total_quantity.set_active(self.config.getboolean('view', 'show_total_quantity'))
		
		checkbutton_show_total_price = widgets.get_widget('checkbutton_show_total_price')
		checkbutton_show_total_price.set_active(self.config.getboolean('view', 'show_total_price'))
		
		## Set default option for Print Tab
		fontbutton_print = widgets.get_widget('fontbutton_print')
		fontbutton_print.set_font_name(self.config.get('print', 'font_name'))
		
		colorbutton_print = widgets.get_widget('colorbutton_print')
		colorbutton_print.set_color(gtk.gdk.color_parse(self.config.get('print', 'color')))
		
		checkbutton_show_title = widgets.get_widget('checkbutton_show_title')
		checkbutton_show_title.set_active(self.config.getboolean('print', 'show_title'))
		
		checkbutton_show_page_number = widgets.get_widget('checkbutton_show_page_number')
		checkbutton_show_page_number.set_active(self.config.getboolean('print', 'show_page_number'))
		
		spinbutton_print_table_border = widgets.get_widget('spinbutton_print_table_border')
		spinbutton_print_table_border.set_value(self.config.getint('print', 'table_border'))
		
		checkbutton_show_print_column_category = widgets.get_widget('checkbutton_show_print_column_category')
		checkbutton_show_print_column_category.set_active(self.config.getboolean('print', 'show_column_category'))
		
		checkbutton_show_print_column_subcategory = widgets.get_widget('checkbutton_show_print_column_subcategory')
		checkbutton_show_print_column_subcategory.set_active(self.config.getboolean('print', 'show_column_subcategory'))
		
		checkbutton_show_print_column_price = widgets.get_widget('checkbutton_show_print_column_price')
		checkbutton_show_print_column_price.set_active(self.config.getboolean('print', 'show_column_price'))
		
		checkbutton_show_print_column_quantity = widgets.get_widget('checkbutton_show_print_column_quantity')
		checkbutton_show_print_column_quantity.set_active(self.config.getboolean('print', 'show_column_quantity'))
		
		checkbutton_show_print_quantity_price_total = widgets.get_widget('checkbutton_show_print_quantity_price_total')
		checkbutton_show_print_quantity_price_total.set_active(self.config.getboolean('print', 'show_line_total'))
		
		events = {
			"on_entry_devise_changed" : self.on_change_devise,
			"on_checkbutton_show_column_category_in_cart_toggled" : self.on_change_check_button_show_column_category_in_cart,
			"on_checkbutton_show_column_subcategory_in_cart_toggled" : self.on_change_check_button_show_column_subcategory_in_cart,
			"on_checkbutton_show_column_price_toggled" : self.on_change_check_button_show_column_price,
			"on_checkbutton_show_column_quantity_toggled" : self.on_change_check_button_show_column_quantity,
			"on_checkbutton_show_total_quantity_toggled" : self.on_change_check_button_show_total_quantity,
			"on_checkbutton_show_total_price_toggled" : self.on_change_check_button_total_price,
			"on_fontbutton_print_font_set" : self.on_change_print_police,
			"on_colorbutton_print_color_set" : self.on_change_print_color,
			"on_checkbutton_show_title_toggled" : self.on_change_checkbutton_show_title,
			"on_checkbutton_show_page_number_toggled" : self.on_change_show_page_number,
			"on_spinbutton_print_table_border_value_changed" : self.on_change_table_border,
			"on_checkbutton_show_print_column_price_toggled" : self.on_change_show_print_column_price,
			"on_checkbutton_show_print_column_quantity_toggled" : self.on_change_show_print_column_quantity,
			"on_checkbutton_show_print_quantity_price_total_toggled" : self.on_change_show_line_total
		}
		widgets.signal_autoconnect(events)
		
		dialog.run()
		self.refresh_display()
		dialog.destroy()
	
	#
	################## End Preferences Dialog ##########################
	
	################## Start Print Operation ###########################
	#
	#	Begin print
	def begin_print(self, operation, context) :
		height = context.get_height()
		
		color = gtk.gdk.color_parse(self.config.get('print', 'color'))
		self.color = (color.red/65535.0, color.green/65535.0, color.blue/65535.0)
		
		font_desc = pango.FontDescription(self.config.get('print', 'font_name'))
		self.size_police = font_desc.get_size()/pango.SCALE
		
		self.layout = context.create_pango_layout()
		self.layout.set_font_description(font_desc)
		
		layout_width,layout_height = self.layout.get_size()
		text_height = layout_height / pango.SCALE
		
		self.nb_lines = int(math.floor(height/(text_height+10)))
		
		self.purchases = self.db.get_all_purchase(self.current_cart_id)
		self.nb_purchases = len(self.purchases)
		
		## remove head page and head table
		self.nb_articles_per_page = self.nb_lines - 2
		self.nb_pages = int((self.nb_purchases + 3) / self.nb_lines) + 1
		
		operation.set_n_pages(self.nb_pages)
		
		self.total_price = 0.00
		self.total_quantity = 0
	
	#
	#	Draw page
	def draw_page(self, operation, context, page_nr) :
		cr = context.get_cairo_context()
		cr.set_source_rgb(self.color[0], self.color[1], self.color[2])
		cr.set_line_width(self.config.getint('print', 'table_border'))
		
		width = context.get_width()
		height = context.get_height()
		
		layout_width,layout_height = self.layout.get_size()
		text_height = layout_height / pango.SCALE
		# space between the cell wall and the cell content
		cell_padding = 5
		# cell height
		cell_height = text_height + 2 * cell_padding
		# number line to write (/!\ start by zero)
		nb_line = 0
		#
		# To create a rectangle : cr.rectangle(0, cell_height * nb_line, width, cell_height)
		# To write text in cell : cr.move_to(cell_padding, cell_height * nb_line + cell_padding)
		#
		
		#cr.rectangle(0, cell_height * nb_line, width, cell_height)
		self.layout.set_text("Glda version %s"%self.appInfos["version"])
		self.layout.set_alignment(pango.ALIGN_LEFT)
		cr.move_to(cell_padding, cell_height * nb_line + cell_padding)
		cr.show_layout(self.layout)
		
		page = "%d / %d"%(page_nr+1, self.nb_pages)
		self.layout.set_text(page)
		self.layout.set_alignment(pango.ALIGN_LEFT)
		cr.move_to(width - len(page)*self.size_police, cell_height * nb_line + cell_padding)
		cr.show_layout(self.layout)
		
		## Start table head
		nb_line += 1
		cr.rectangle(0, cell_height * nb_line, width*0.60, cell_height)
		self.layout.set_text("Articles")
		self.layout.set_alignment(pango.ALIGN_LEFT)
		cr.move_to(cell_padding, cell_height * nb_line + cell_padding)
		cr.show_layout(self.layout)
		
		cr.rectangle(width*0.60, cell_height * nb_line, width*0.20, cell_height)
		self.layout.set_text("Quantité")
		self.layout.set_alignment(pango.ALIGN_LEFT)
		cr.move_to(cell_padding + width*0.60, cell_height * nb_line + cell_padding)
		cr.show_layout(self.layout)
		
		cr.rectangle(width*0.80, cell_height * nb_line, width*0.20, cell_height)
		self.layout.set_text("Prix")
		self.layout.set_alignment(pango.ALIGN_LEFT)
		cr.move_to(cell_padding + width*0.80, cell_height * nb_line + cell_padding)
		cr.show_layout(self.layout)
		## End table head
		
		## Start list of articles
		purchases = self.db.get_all_purchase_with_limit(self.current_cart_id, self.nb_articles_per_page, page_nr*(self.nb_articles_per_page))
		for purchase in purchases :
			article = self.db.get_article(purchase['id_article'])
			
			nb_line += 1
			cr.rectangle(0, cell_height * nb_line, width*0.60, cell_height)
			self.layout.set_text(article['name'])
			self.layout.set_alignment(pango.ALIGN_LEFT)
			cr.move_to(cell_padding, cell_height * nb_line + cell_padding)
			cr.show_layout(self.layout)
			
			cr.rectangle(width*0.60, cell_height * nb_line, width*0.20, cell_height)
			self.layout.set_text("%d"%purchase['quantity'])
			self.layout.set_alignment(pango.ALIGN_LEFT)
			cr.move_to(cell_padding + width*0.60, cell_height * nb_line + cell_padding)
			cr.show_layout(self.layout)
			
			cr.rectangle(width*0.80, cell_height * nb_line, width*0.20, cell_height)
			self.layout.set_text("%.2f"%(article['price']*purchase['quantity']))
			self.layout.set_alignment(pango.ALIGN_LEFT)
			cr.move_to(cell_padding + width*0.80, cell_height * nb_line + cell_padding)
			cr.show_layout(self.layout)
			
			self.total_price += article['price']*purchase['quantity']
			self.total_quantity += purchase['quantity']
		
		if self.nb_pages == page_nr+1 :
			nb_line += 1
			#cr.rectangle(0, cell_height * nb_line, width*0.60, cell_height)
			self.layout.set_text("Totaux")
			self.layout.set_alignment(pango.ALIGN_LEFT)
			cr.move_to(cell_padding, cell_height * nb_line + cell_padding)
			cr.show_layout(self.layout)
			
			cr.rectangle(width*0.60, cell_height * nb_line, width*0.20, cell_height)
			self.layout.set_text("%d"%self.total_quantity)
			self.layout.set_alignment(pango.ALIGN_LEFT)
			cr.move_to(cell_padding + width*0.60, cell_height * nb_line + cell_padding)
			cr.show_layout(self.layout)
			
			devise = self.config.get('general', 'devise')
			cr.rectangle(width*0.80, cell_height * nb_line, width*0.20, cell_height)
			self.layout.set_text("%.2f %s"%(self.total_price, devise))
			self.layout.set_alignment(pango.ALIGN_LEFT)
			cr.move_to(cell_padding + width*0.80, cell_height * nb_line + cell_padding)
			cr.show_layout(self.layout)
		
		cr.stroke()
	
	#
	#	On print
	def on_print(self, widget, data=None) :
		print_operation = gtk.PrintOperation()
		
		print_operation.connect("begin_print", self.begin_print)
		print_operation.connect("draw_page", self.draw_page)
		
		mainWindow = self.widgets.get_widget('mainWindow')
		print_operation.run(gtk.PRINT_OPERATION_ACTION_PRINT_DIALOG, mainWindow)
	
	################## End Print Operation #############################
	
	################## Start Manage Store Dialog #######################
	#
	def on_add_category(self, menu_item) :
		widgets = gtk.glade.XML(GLDA_GLADE_FILE, "dialog_add_category")
		dialog = widgets.get_widget("dialog_add_category")
		dialog.set_transient_for(self.widgets_dialog_store.get_widget("storeDialog"))
		dialog.add_buttons(gtk.STOCK_OK, gtk.RESPONSE_OK, gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
		
		if dialog.run() == gtk.RESPONSE_OK:
			category_name = widgets.get_widget("entry_category_name").get_text()
			self.db.set_category(category_name)
			self.refresh_store()
		
		dialog.destroy()
	
	def on_modify_category(self, menu_item, id_category) :
		widgets = gtk.glade.XML(GLDA_GLADE_FILE, "dialog_add_category")
		dialog = widgets.get_widget("dialog_add_category")
		dialog.set_transient_for(self.widgets_dialog_store.get_widget("storeDialog"))
		dialog.add_buttons(gtk.STOCK_OK, gtk.RESPONSE_OK, gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
		
		category = self.db.get_category(id_category)
		widgets.get_widget("entry_category_name").set_text(category["name"])
		
		if dialog.run() == gtk.RESPONSE_OK:
			category_name = widgets.get_widget("entry_category_name").get_text()
			self.db.update_category(id_category, category_name)
			self.refresh_store()
		
		dialog.destroy()
	
	def on_remove_category(self, menu_item, id_category) :
		widgets = gtk.glade.XML(GLDA_GLADE_FILE, "dialog_msg")
		dialog = widgets.get_widget("dialog_msg")
		dialog.set_transient_for(self.widgets_dialog_store.get_widget("storeDialog"))
		dialog.add_buttons(gtk.STOCK_OK, gtk.RESPONSE_OK, gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
		
		category = self.db.get_category(id_category)
		widgets.get_widget("label_msg").set_text("Êtes-vous sûr de supprimer la sous categorie : %s?"%category["name"])
		
		widgets.get_widget("label_msg1").set_markup("<b>Les sous categories et les articles contenus dans la categorie seront supprimée aussi!!!</b>")
		
		if dialog.run() == gtk.RESPONSE_OK :
			all_subcategory = self.db.get_all_subcategory_by_category(id_category)
			for subcategory in all_subcategory :
				articles = self.db.get_all_articles_by_subcategory(subcategory["id"])
				for article in articles :
					purchases = self.db.get_purchases_by_article(article["id"])
					for purchase in purchases :
						self.db.remove_purchase(purchase["id"])
					self.db.remove_article(article["id"])
				self.db.remove_subcategory(subcategory["id"])
			self.db.remove_category(id_category)
			self.refresh_store()
		
		dialog.destroy()
	
	def on_add_subcategory(self, menu_item, id_category) :
		widgets = gtk.glade.XML(GLDA_GLADE_FILE, "dialog_add_subcategory")
		dialog = widgets.get_widget("dialog_add_subcategory")
		dialog.set_transient_for(self.widgets_dialog_store.get_widget("storeDialog"))
		dialog.add_buttons(gtk.STOCK_OK, gtk.RESPONSE_OK, gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
		
		category = self.db.get_category(id_category)
		widgets.get_widget("entry_category_name_for_subcategory").set_text(category["name"])
		
		if dialog.run() == gtk.RESPONSE_OK:
			subcategory_name = widgets.get_widget("entry_subcategory_name").get_text()
			self.db.set_subcategory(id_category, subcategory_name)
			self.refresh_store()
		
		dialog.destroy()
	
	def on_modify_subcategory(self, menu_item, id_category, id_subcategory) :
		widgets = gtk.glade.XML(GLDA_GLADE_FILE, "dialog_add_subcategory")
		dialog = widgets.get_widget("dialog_add_subcategory")
		dialog.set_transient_for(self.widgets_dialog_store.get_widget("storeDialog"))
		dialog.add_buttons(gtk.STOCK_OK, gtk.RESPONSE_OK, gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
		
		category = self.db.get_category(id_category)
		widgets.get_widget("entry_category_name_for_subcategory").set_text(category["name"])
		
		subcategory = self.db.get_subcategory(id_subcategory)
		widgets.get_widget("entry_subcategory_name").set_text(subcategory["name"])
		
		if dialog.run() == gtk.RESPONSE_OK:
			subcategory_name = widgets.get_widget("entry_subcategory_name").get_text()
			self.db.update_subcategory(id_subcategory, subcategory_name)
			self.refresh_store()
		
		dialog.destroy()
	
	def on_remove_subcategory(self, menu_item, id_subcategory) :
		widgets = gtk.glade.XML(GLDA_GLADE_FILE, "dialog_msg")
		dialog = widgets.get_widget("dialog_msg")
		dialog.set_transient_for(self.widgets_dialog_store.get_widget("storeDialog"))
		dialog.add_buttons(gtk.STOCK_OK, gtk.RESPONSE_OK, gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
		
		subcategory = self.db.get_subcategory(id_subcategory)
		widgets.get_widget("label_msg").set_text("Êtes-vous sûr de supprimer la sous categorie : %s?"%subcategory["name"])
		
		widgets.get_widget("label_msg1").set_markup("<b>Les articles contenus dans la sous categorie seront supprimée aussi!!!</b>")
		
		if dialog.run() == gtk.RESPONSE_OK:
			articles = self.db.get_all_articles_by_subcategory(id_subcategory)
			for article in articles :
				purchases = self.db.get_purchases_by_article(article["id"])
				for purchase in purchases :
					self.db.remove_purchase(purchase["id"])
				self.db.remove_article(article["id"])
			self.db.remove_subcategory(id_subcategory)
			self.refresh_store()
		
		dialog.destroy()
	
	def on_add_article(self, menu_item, id_category, id_subcategory) :
		widgets = gtk.glade.XML(GLDA_GLADE_FILE, "dialog_add_article")
		dialog = widgets.get_widget("dialog_add_article")
		dialog.set_transient_for(self.widgets_dialog_store.get_widget("storeDialog"))
		dialog.add_buttons(gtk.STOCK_OK, gtk.RESPONSE_OK, gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
		
		category = self.db.get_category(id_category)
		widgets.get_widget("entry_category_name_for_article").set_text(category["name"])
		
		subcategory = self.db.get_subcategory(id_subcategory)
		widgets.get_widget("entry_subcategory_name_for_article").set_text(subcategory["name"])
		
		if dialog.run() == gtk.RESPONSE_OK:
			article_name = widgets.get_widget("entry_article_name").get_text()
			article_price = widgets.get_widget("spinbutton_article_price").get_value()
			self.db.set_article(id_category, id_subcategory, article_name,article_price)
			self.refresh_store()
		
		dialog.destroy()
	
	def on_modify_article(self, menu_item, id_category, id_subcategory, id_article) :
		widgets = gtk.glade.XML(GLDA_GLADE_FILE, "dialog_add_article")
		dialog = widgets.get_widget("dialog_add_article")
		dialog.set_transient_for(self.widgets_dialog_store.get_widget("storeDialog"))
		dialog.add_buttons(gtk.STOCK_OK, gtk.RESPONSE_OK, gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
		
		category = self.db.get_category(id_category)
		widgets.get_widget("entry_category_name_for_article").set_text(category["name"])
		
		subcategory = self.db.get_subcategory(id_subcategory)
		widgets.get_widget("entry_subcategory_name_for_article").set_text(subcategory["name"])
		
		article = self.db.get_article(id_article)
		widgets.get_widget("entry_article_name").set_text(article["name"])
		widgets.get_widget("spinbutton_article_price").set_value(article["price"])
		
		if dialog.run() == gtk.RESPONSE_OK:
			article_name = widgets.get_widget("entry_article_name").get_text()
			article_price = widgets.get_widget("spinbutton_article_price").get_value()
			self.db.update_article(id_article, article_name, article_price)
			self.refresh_store()
		
		dialog.destroy()
	
	def on_remove_article(self, menu_item, id_article) :
		widgets = gtk.glade.XML(GLDA_GLADE_FILE, "dialog_msg")
		dialog = widgets.get_widget("dialog_msg")
		dialog.set_transient_for(self.widgets_dialog_store.get_widget("storeDialog"))
		dialog.add_buttons(gtk.STOCK_OK, gtk.RESPONSE_OK, gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
		
		article = self.db.get_article(id_article)
		widgets.get_widget("label_msg").set_text("Êtes-vous sûr de supprimer l'article : %s?"%article["name"])
		
		widgets.get_widget("label_msg1").set_markup("<b>L'article sera aussi supprimé du panier qui le contient!!!</b>")
		
		if dialog.run() == gtk.RESPONSE_OK:
			purchases = self.db.get_purchases_by_article(id_article)
			for purchase in purchases :
				self.db.remove_purchase(purchase["id"])
			self.db.remove_article(id_article)
			self.refresh_store()
		
		dialog.destroy()
	
	def on_button_press_event(self, widget, event) :
		if event.button == 3 and event.type == gtk.gdk.BUTTON_PRESS :
			treeview = self.widgets_dialog_store.get_widget("treeview_store")
			tree_selection = treeview.get_selection()
			
			tree_store, iter_selected = tree_selection.get_selected()
			
			menu = gtk.Menu()
			
			if iter_selected == None :
				menu_item = gtk.MenuItem("Ajouter une categorie")
				menu_item.connect("activate", self.on_add_category)
				menu.append(menu_item)
			
			else :
				id_category, id_subcategory, id_article = tree_store.get(iter_selected,
																		COLUMN_STORE_CATEGORY_ID,
																		COLUMN_STORE_SUBCATEGORY_ID,
																		COLUMN_STORE_ARTICLE_ID
				)
				
				## category is selected
				if id_category != 0 and id_subcategory == 0 and id_article == 0 :
					menu_item = gtk.MenuItem("Ajouter une categorie")
					menu_item.connect("activate", self.on_add_category)
					menu.append(menu_item)
					
					menu_item = gtk.MenuItem("Ajouter une sous categorie")
					menu_item.connect("activate", self.on_add_subcategory, id_category)
					menu.append(menu_item)
					
					menu_item = gtk.MenuItem("Modifier la categorie")
					menu_item.connect("activate", self.on_modify_category, id_category)
					menu.append(menu_item)
					
					menu_item = gtk.MenuItem("Supprimer la categorie")
					menu_item.connect("activate", self.on_remove_category, id_category)
					menu.append(menu_item)
				
				## subcategory is selected
				elif id_category != 0 and id_subcategory != 0 and id_article == 0 :
					menu_item = gtk.MenuItem("Ajouter une article")
					menu_item.connect("activate", self.on_add_article, id_category, id_subcategory)
					menu.append(menu_item)
					
					menu_item = gtk.MenuItem("Modifier la sous categorie")
					menu_item.connect("activate", self.on_modify_subcategory, id_category, id_subcategory)
					menu.append(menu_item)
					
					menu_item = gtk.MenuItem("Supprimer la sous categorie")
					menu_item.connect("activate", self.on_remove_subcategory, id_subcategory)
					menu.append(menu_item)
				
				## article is selected
				else :
					menu_item = gtk.MenuItem("Modifier l'article")
					menu_item.connect("activate", self.on_modify_article, id_category, id_subcategory, id_article)
					menu.append(menu_item)
					
					menu_item = gtk.MenuItem("Supprimer l'article")
					menu_item.connect("activate", self.on_remove_article, id_article)
					menu.append(menu_item)
			
			menu.popup(None, None, None, event.button, event.time)
			menu.show_all()
	
	def treeview_store(self) :
		tree_store = gtk.TreeStore(str, str, int, int, int)
		treeview = self.widgets_dialog_store.get_widget("treeview_store")
		treeview.set_model(tree_store)
		treeview.set_search_column(COLUMN_STORE_CATEGORY)
		treeview.connect("button-press-event", self.on_button_press_event)
		
		tree_selection = treeview.get_selection()
		tree_selection.set_mode(gtk.SELECTION_SINGLE)
		
		cellRenderer = gtk.CellRendererText()
		column = gtk.TreeViewColumn("Categories", cellRenderer)
		column.add_attribute(cellRenderer, "text", COLUMN_STORE_CATEGORY)
		column.set_sort_column_id(COLUMN_STORE_PRICE)
		treeview.append_column(column)
		
		#~ adjustment = gtk.Adjustment(0.00, 0.00, 100.00, 1.00, 10.00, 00.00)        
		#~ cellRenderer = gtk.CellRendererSpin()
		#~ cellRenderer.set_property("editable", True)
		#~ cellRenderer.set_property("adjustment", adjustment)
		#~ cellRenderer.connect("edited", self.on_change_store_price, tree_store)
		
		cellRenderer = gtk.CellRendererText()
		column = gtk.TreeViewColumn("Prix", cellRenderer)
		column.add_attribute(cellRenderer, "text", COLUMN_STORE_PRICE)
		column.set_sort_column_id(COLUMN_STORE_PRICE)
		treeview.append_column(column)
		
		cellRenderer = gtk.CellRendererText()
		column = gtk.TreeViewColumn("id-category", cellRenderer)
		column.add_attribute(cellRenderer, "text", COLUMN_STORE_CATEGORY_ID)
		column.set_visible(False)
		treeview.append_column(column)
		
		cellRenderer = gtk.CellRendererText()
		column = gtk.TreeViewColumn("id-subcategory", cellRenderer)
		column.add_attribute(cellRenderer, "text", COLUMN_STORE_SUBCATEGORY_ID)
		column.set_visible(False)
		treeview.append_column(column)
		
		cellRenderer = gtk.CellRendererText()
		column = gtk.TreeViewColumn("id-articles", cellRenderer)
		column.add_attribute(cellRenderer, "text", COLUMN_STORE_ARTICLE_ID)
		column.set_visible(False)
		treeview.append_column(column)
	
	def refresh_store(self) :
		treeview = self.widgets_dialog_store.get_widget("treeview_store")
		tree_store = treeview.get_model()
		tree_store.clear()
		
		all_category = self.db.get_all_category()
		for category in all_category :
			iter = tree_store.append(None)
			tree_store.set(iter,
						COLUMN_STORE_CATEGORY, category["name"],
						COLUMN_STORE_CATEGORY_ID, category["id"],
						COLUMN_STORE_SUBCATEGORY_ID, 0,
						COLUMN_STORE_ARTICLE_ID, 0
			)
			
			all_subcategory = self.db.get_all_subcategory_by_category(category["id"])
			for subcategory in all_subcategory :
				iter_child1 = tree_store.append(iter)
				tree_store.set(iter_child1,
							COLUMN_STORE_CATEGORY, subcategory["name"],
							COLUMN_STORE_CATEGORY_ID, subcategory["id_category"],
							COLUMN_STORE_SUBCATEGORY_ID, subcategory["id"],
							COLUMN_STORE_ARTICLE_ID, 0
				)
				
				all_articles = self.db.get_all_articles_by_subcategory(subcategory["id"])
				for article in all_articles :
					iter_child2 = tree_store.append(iter_child1)
					tree_store.set(iter_child2,
								COLUMN_STORE_CATEGORY, article["name"],
								COLUMN_STORE_PRICE, "%.2f"%article["price"],
								COLUMN_STORE_CATEGORY_ID, article["id_category"],
								COLUMN_STORE_SUBCATEGORY_ID, article["id_subcategory"],
								COLUMN_STORE_ARTICLE_ID, article["id"]
					)
	
	def on_manage_store(self, widget, data=None) :
		self.widgets_dialog_store = gtk.glade.XML(GLDA_GLADE_FILE, "storeDialog")
		dialog = self.widgets_dialog_store.get_widget("storeDialog")
		dialog.set_transient_for(self.widgets.get_widget('mainWindow'))
		
		self.treeview_store()
		self.refresh_store()
		
		dialog.run()
		dialog.destroy()
		
		self.refresh_category()
		self.refresh_cart()
	
	################## Start Manage Store Dialog #######################
	
	################## Start About Dialog ##############################
	#
	def on_about(self, widget, data=None) :
		try :
			file = open(self.appInfos['license'], 'r')
			license = file.read()
		except :
			license = "GPL version 3 ou superieure"
		
		logo = gtk.gdk.pixbuf_new_from_file(self.appInfos['icon'])
		
		widgets = gtk.glade.XML(GLDA_GLADE_FILE, "aboutDialog")
		dialog = widgets.get_widget('aboutDialog')
		dialog.set_transient_for(self.widgets.get_widget('mainWindow'))
		dialog.set_authors(self.appInfos['authors'])
		dialog.set_documenters(self.appInfos['documenters'])
		dialog.set_artists(self.appInfos['artists'])
		dialog.set_name(self.appInfos['name'])
		dialog.set_version(self.appInfos['version'])
		dialog.set_comments("Glda (Gtk Liste D'Achat) est un utilitaire pour faire sa liste d'achat.")
		dialog.set_copyright("(c) 2008 - 2011 Tuxgasy")
		dialog.set_website(self.appInfos['website'])
		dialog.set_logo(logo)
		dialog.set_license(license)
		
		dialog.run()
		dialog.destroy()
	
	################## End About Dialog ################################

################## Start Main ##########################################
#
if __name__ == "__main__" :
	config = Config()
	
	try :
		### old config is exist?
		list_file = config.get("file", "file_list_product")
		## convert old xml database to sqlite database
		xml2sqlite(list_file, "%s/glda.db"%DIR_USER_CONFIG)
		## rename old file config
		os.renames(FILE_USER_CONFIG, "%s.bak"%FILE_USER_CONFIG)
		## regenerate the new config
		config = Config()
	## cool have the new config
	except ConfigParser.NoSectionError :
		pass
	
	db = Db(config.get('database', 'db_file'))
	mainWindow = mainWindow(config, db, appInfos)
	gtk.main()
